@@ -27,7 +27,6 @@
 #include <qstring.h>
 #include <qobject.h>
 #include <qfile.h>
-#include <qvaluelist.h>
 
 #include <torrent/globals.h>
 #include <util/log.h>
@@ -35,32 +34,42 @@
 #include <util/functions.h>
 #include <util/constants.h>
 		
-#include <torrent/bnode.h>
-#include <torrent/bdecoder.h>
+#include <bcodec/bnode.h>
+#include <bcodec/bdecoder.h>
 
 #include <interfaces/coreinterface.h>
+#include "scanfolderpluginsettings.h"
 
 using namespace bt;
 
 namespace kt
 {
 
-	ScanFolder::ScanFolder(CoreInterface* core, QString& dir, LoadedTorrentAction action, bool openSilently)
-			: m_core(core), m_dir(0), m_loadedAction(action), m_openSilently(openSilently)
+	ScanFolder::ScanFolder(CoreInterface* core, const QString& dir, LoadedTorrentAction action)
+			: m_core(core), m_dir(0), m_loadedAction(action)
 	{
+		bt::Out(SYS_SNF|LOG_NOTICE) << "ScanFolder : scanning " << dir << endl;
 		m_dir = new KDirLister();
 
-		if(!m_dir->openURL(dir)) {
+		if(!m_dir->openUrl(dir,KDirLister::Keep)) 
+		{
 			m_valid = false;
+			bt::Out(SYS_SNF|LOG_NOTICE) << "ScanFolder : m_dir->openUrl failed " << endl;
 			return;
-		} else
+		} 
+		else
 			m_valid = true;
 
 		m_dir->setShowingDotFiles(true);
 
 		connect(m_dir, SIGNAL(newItems( const KFileItemList& )), this, SLOT(onNewItems( const KFileItemList& )));
-		connect(m_core, SIGNAL(loadingFinished( const KURL&, bool, bool )), this, SLOT(onLoadingFinished( const KURL&, bool, bool )));
+		connect(m_core, SIGNAL(loadingFinished( const KUrl&, bool, bool )), this, SLOT(onLoadingFinished( const KUrl&, bool, bool )));
 		connect(&m_incomplePollingTimer,SIGNAL(timeout()),this,SLOT(onIncompletePollingTimeout()));
+		
+		if( (m_loadedAction == moveAction) && !QDir(dir + i18n("loaded")).exists())
+			KIO::mkdir(dir + i18n("loaded"));
+		
+		m_dir->updateDirectory(KUrl(dir));
 	}
 
 
@@ -72,31 +81,47 @@ namespace kt
 
 	void ScanFolder::onNewItems(const KFileItemList& items)
 	{
-		KFileItemList list = items;
-		KFileItem* file;
-		for(file=list.first(); file; file=list.next()) 
+		bool rec = ScanFolderPluginSettings::recursive();
+		foreach (const KFileItem &file, items)
 		{
-			QString name = file->name();
-			QString dirname = m_dir->url().path();
-			QString filename = dirname + bt::DirSeparator() + name;
+			QString name = file.name();
+			QString filename = file.url().path();
+			
+			if (file.isDir() && name != i18n("loaded"))
+			{
+				// watch subdirectories, but not the loaded directory
+				m_dir->openUrl(file.url(),KDirLister::Keep); 
+				m_dir->updateDirectory(file.url());
+				continue;
+			}
 
-			if(!name.endsWith(".torrent"))
+			if (!name.endsWith(".torrent"))
+				continue;
+			
+			QString dirname = QFileInfo(filename).absolutePath();
+			if (!dirname.endsWith(bt::DirSeparator()))
+				dirname += bt::DirSeparator();
+			
+			QString root_dir = m_dir->url().path();
+			if (!root_dir.endsWith(bt::DirSeparator()))
+				root_dir += bt::DirSeparator();
+			
+			if (!rec && root_dir != dirname) // if recusive is disabled root_dir must be equal to dirnam
 				continue;
 
-			if(name.startsWith(".")) 
+			if (name.startsWith("."))
 			{
 				//Check if corresponding torrent exists
-				if(!QFile::exists(m_dir->url().path() + bt::DirSeparator() + name.right(name.length() - 1)) && (m_loadedAction == defaultAction))
+				if(!QFile::exists(dirname + name.right(name.length() - 1)) && (m_loadedAction == defaultAction))
 					QFile::remove(filename);
 
 				continue;
 			}
 
-			KURL source;
-			source.setPath(filename);
+			KUrl source(filename);
 
 			//If torrent has it's hidden complement - skip it.
-			if(QFile::exists(dirname + "/." + name))
+			if (QFile::exists(dirname + "/." + name))
 				continue;
 			
 			if (incomplete(source))
@@ -108,7 +133,7 @@ namespace kt
 				{
 					// first URL so start the poll timer
 					// lets poll every 10 seconds
-					m_incomplePollingTimer.start(10000,false);
+					m_incomplePollingTimer.start(10000);
 				}
 			}
 			else
@@ -117,98 +142,99 @@ namespace kt
 				//Add pending entry...
 				m_pendingURLs.push_back(source);
 				
+				QString group;
+				if (ScanFolderPluginSettings::addToGroup())
+					group = ScanFolderPluginSettings::group();
 				//Load torrent
-				if(m_openSilently)
-					m_core->loadSilently(source);
+				if (ScanFolderPluginSettings::openSilently())
+					m_core->loadSilently(source,group);
 				else
-					m_core->load(source);
+					m_core->load(source,group);
 			}
 		}
 	}
 	
-	void ScanFolder::onLoadingFinished(const KURL & url, bool success, bool canceled)
+	void ScanFolder::onLoadingFinished(const KUrl & url, bool success, bool canceled)
 	{
 		if(m_pendingURLs.empty() || !success)
 			return;
 		
-		//search for entry
-		QValueList<KURL>::iterator it = m_pendingURLs.find(url);
 		
-		//if no entry is found than this torrent was not started by this plugin so - quit
-		if(it == m_pendingURLs.end())
+		// if no entry is found than this torrent was not started by this plugin so - quit
+		if (!m_pendingURLs.contains(url))
 			return;
 		
 		//remove this entry
-		m_pendingURLs.erase(it);
+		m_pendingURLs.removeAll(url);
 		
-		if(canceled)
-			return;
+		QString name = url.fileName();
+		QString dirname = QFileInfo(url.path()).absolutePath();
+		if (!dirname.endsWith(bt::DirSeparator()))
+			dirname += bt::DirSeparator();
 		
-		QString name = url.filename(false);
-		QString dirname = m_dir->url().path();
-		QString filename = dirname + "/" + name;
-		KURL destination(dirname + "/" + i18n("loaded") + "/" + name);
+		QString filename = dirname + name;
 		
-		switch(m_loadedAction) {
+		QString root_dir = m_dir->url().path();
+		if (!root_dir.endsWith(bt::DirSeparator()))
+			root_dir += bt::DirSeparator();
+		KUrl destination(root_dir + i18n("loaded") + bt::DirSeparator() + name);
+		
+		switch(m_loadedAction) 
+		{
 			case deleteAction:
 					//If torrent has it's hidden complement - remove it too.
-				if(QFile::exists(dirname + "/." + name))
-					QFile::remove(dirname + "/." + name);
+				if(QFile::exists(dirname + "." + name))
+					QFile::remove(dirname + "." + name);
 					// 				Out() << "Deleting: " << name.ascii() << endl;
 				QFile::remove(filename);
 				break;
 			case moveAction:
 					// 				Out() << "Moving: " << name.ascii() << endl;
 					//If torrent has it's hidden complement - remove it too.
-				if(QFile::exists(dirname + "/." + name))
-					QFile::remove(dirname + "/." + name);
+				if(QFile::exists(dirname + "." + name))
+					QFile::remove(dirname + "." + name);
 
 				// NetAccess considered harmfull !!!
 				KIO::file_move(url, destination);
 				break;
 			case defaultAction:
-				QFile f(dirname + "/." + name);
-				f.open(IO_WriteOnly);
+				QFile f(dirname + "." + name);
+				f.open(QIODevice::WriteOnly);
 				f.close();
 				break;
 		}
 	}
 
-	void ScanFolder::setOpenSilently(bool theValue)
-	{
-		m_openSilently = theValue;
-	}
-
 	void ScanFolder::setLoadedAction(const LoadedTorrentAction& theValue)
 	{
 		m_loadedAction = theValue;
 
-		QDir tmp(m_dir->url().path());
+		QString path = m_dir->url().path();
+		if (!path.endsWith(bt::DirSeparator()))
+			path += bt::DirSeparator();
+		
+		QDir tmp(path);
 
-		if( (m_loadedAction == moveAction) && !tmp.exists(i18n("loaded"), false))
-			tmp.mkdir(i18n("loaded"), false);
+		if( (m_loadedAction == moveAction) && !QDir(path + i18n("loaded")).exists())
+			KIO::mkdir(path + i18n("loaded"));
 	}
 
 	void ScanFolder::setFolderUrl(QString& url)
 	{
-		if(!m_dir->openURL(url)) {
-			m_valid = false;
-			return;
-		} else
-			m_valid = true;
+		m_valid = m_dir->openUrl(url);
 	}
 	
-	bool ScanFolder::incomplete(const KURL & src)
+	bool ScanFolder::incomplete(const KUrl & src)
 	{
 		// try to decode file, if it is syntactically correct, we can try to load it
 		QFile fptr(src.path());
-		if (!fptr.open(IO_ReadOnly))
+		if (!fptr.open(QIODevice::ReadOnly))
 			return false;
 		
 		try
 		{
-			QByteArray data(fptr.size());
-			fptr.readBlock(data.data(),fptr.size());
+			QByteArray data = fptr.readAll();
+			
 			bt::BDecoder dec(data,false);
 			bt::BNode* n = dec.decode();
 			if (n)
@@ -234,9 +260,9 @@ namespace kt
 	void ScanFolder::onIncompletePollingTimeout()
 	{
 		bt::Out(SYS_SNF|LOG_NOTICE) << "ScanFolder : checking incomplete files" << endl; 
-		for (QValueList<KURL>::iterator i = m_incompleteURLs.begin(); i != m_incompleteURLs.end();)
+		for (QList<KUrl>::iterator i = m_incompleteURLs.begin(); i != m_incompleteURLs.end();)
 		{
-			KURL source = *i;
+			KUrl source = *i;
 			if (!bt::Exists(source.path()))
 			{
 				// doesn't exist anymore, so throw out of list
@@ -248,11 +274,15 @@ namespace kt
 				//Add pending entry...
 				m_pendingURLs.push_back(source);
 				
+				QString group;
+				if (ScanFolderPluginSettings::addToGroup())
+					group = ScanFolderPluginSettings::group();
+				
 				//Load torrent
-				if(m_openSilently)
-					m_core->loadSilently(source);
+				if (ScanFolderPluginSettings::openSilently())
+					m_core->loadSilently(source,group);
 				else
-					m_core->load(source);
+					m_core->load(source,group);
 				
 				// remove from incomplete list
 				i = m_incompleteURLs.erase(i);
@@ -270,4 +300,4 @@ namespace kt
 	}
 }
 
-#include "scanfolder.moc"
\ No newline at end of file
+#include "scanfolder.moc"
