@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005 by Joris Guisson                                   *
+ *   Copyright (C) 2005-2007 by Joris Guisson                              *
  *   joris.guisson@gmail.com                                               *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -15,12 +15,11 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.             *
+ *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.          *
  ***************************************************************************/
 #include <kmessagebox.h>
 #include <kio/job.h>
 #include <kio/jobclasses.h>
-//#include <qfile.h>
 #include <qclipboard.h>
 #include <qapplication.h>
 #include <kio/netaccess.h>
@@ -37,7 +36,7 @@ namespace kt
 {
 	
 	HTMLPart::HTMLPart(QWidget *parent)
-			: KHTMLPart(parent)
+			: KHTMLPart(parent,parent)
 	{
 		setJScriptEnabled(true);
 		setJavaEnabled(true);
@@ -45,8 +44,8 @@ namespace kt
 		setPluginsEnabled(false);
 		setStatusMessagesEnabled(false);
 		KParts::BrowserExtension* ext = this->browserExtension();
-		connect(ext,SIGNAL(openURLRequest(const KURL&,const KParts::URLArgs&)),
-				this,SLOT(openURLRequest(const KURL&, const KParts::URLArgs& )));
+		connect(ext,SIGNAL(openUrlRequest(const KUrl &, const KParts::OpenUrlArguments &, const KParts::BrowserArguments &)),
+				this,SLOT(openUrlRequest(const KUrl&,const KParts::OpenUrlArguments & , const KParts::BrowserArguments &)));
 	
 		ext->enableAction("copy",true);
 		ext->enableAction("paste",true);
@@ -66,18 +65,18 @@ namespace kt
 			cb->setText(txt,QClipboard::Clipboard);
 	}
 	
-	void HTMLPart::openURLRequest(const KURL &u,const KParts::URLArgs &)
+	void HTMLPart::openUrlRequest(const KUrl &u, const KParts::OpenUrlArguments & arg, const KParts::BrowserArguments & barg)
 	{
 		if (active_job)
 		{
-			active_job->kill(true);
+			active_job->kill();
 			active_job = 0;
 		}
 		
-		KIO::TransferJob* j = KIO::get(u,false,false);
+		KIO::TransferJob* j = KIO::get(u, KIO::NoReload, KIO::HideProgressInfo);
 		connect(j,SIGNAL(data(KIO::Job*,const QByteArray &)),
-				this,SLOT(dataRecieved(KIO::Job*, const QByteArray& )));
-		connect(j,SIGNAL(result(KIO::Job*)),this,SLOT(jobDone(KIO::Job* )));
+				this,SLOT(dataReceived(KIO::Job*, const QByteArray& )));
+		connect(j,SIGNAL(result(KJob*)),this,SLOT(jobDone(KJob* )));
 		connect(j,SIGNAL(mimetype(KIO::Job*, const QString &)),
 				this,SLOT(mimetype(KIO::Job*, const QString& )));
 	
@@ -96,14 +95,14 @@ namespace kt
 		else
 		{
 			history.pop_back();
-			KURL u = history.back();
-			openURL(u);
+			KUrl u = history.back();
+			openUrl(u);
 			backAvailable(history.count() > 1 ? true : false);
 			
 		}
 	}
 	
-	void HTMLPart::addToHistory(const KURL & url)
+	void HTMLPart::addToHistory(const KUrl & url)
 	{
 		history.append(url);
 		if (history.count() > 1)
@@ -112,14 +111,14 @@ namespace kt
 	
 	void HTMLPart::reload()
 	{
-		openURL(url());
+		openUrl(url());
 	}
 	
-	void HTMLPart::dataRecieved(KIO::Job* job,const QByteArray & data)
+	void HTMLPart::dataReceived(KIO::Job* job,const QByteArray & data)
 	{
 		if (job != active_job)
 		{
-			job->kill(true);
+			job->kill();
 			return;
 		}
 		
@@ -138,18 +137,18 @@ namespace kt
 	{
 		if (job != active_job)
 		{
-			job->kill(true);
+			job->kill();
 			return;
 		}
 	
 		mime_type = mt;
 	}
 	
-	void HTMLPart::jobDone(KIO::Job* job)
+	void HTMLPart::jobDone(KJob* job)
 	{
 		if (job != active_job)
 		{
-			job->kill(true);
+			job->kill();
 			return;
 		}
 		
@@ -164,8 +163,8 @@ namespace kt
 				int ret = KMessageBox::questionYesNoCancel(0,
 						i18n("Do you want to download or save the torrent?"),
 						i18n("Download Torrent"),
-						KGuiItem(i18n("to download", "Download"),"down"),
-						KStdGuiItem::save());
+						KGuiItem(i18n("Download"),"ktorrent"),
+						KStandardGuiItem::save());
 			
 				if (ret == KMessageBox::Yes)
 					openTorrent(curr_url);
@@ -190,7 +189,7 @@ namespace kt
 		}
 		active_job = 0;
 		curr_data.resize(0);
-		curr_url = KURL();
+		curr_url = KUrl();
 		mime_type = QString::null;
 	}
 }
