@@ -17,18 +17,20 @@
  *   Free Software Foundation, Inc.,                                       *
  *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.             *
  ***************************************************************************/
+#include <QDir>
+#include <QSocketNotifier>
 #include <qtimer.h>
-#include <qcstring.h>
 #include <qdatetime.h>
+#include <kcodecs.h>
 #include <kapplication.h>
 #include <kgenericfactory.h>
 #include <kglobal.h>
 #include <kstandarddirs.h>
-#include <kmdcodec.h>
-#include <ktempfile.h>
+#include <ktemporaryfile.h>
+#include <k3streamsocket.h>
+#include <k3resolver.h>
 
 #include <qfileinfo.h>
-#include <qsocket.h>
 #include <qstringlist.h>
 
 #include <interfaces/coreinterface.h>
@@ -38,128 +40,140 @@
 #include <util/fileops.h>
 #include <util/functions.h>
 #include <util/mmapfile.h>
+#include <util/sha1hash.h>
+#include <time.h>
 #include "ktversion.h"
 #include "httpserver.h"
 #include "httpclienthandler.h"
 #include "httpresponseheader.h"
-#include "php_handler.h"
-#include "php_interface.h"
+#include "phphandler.h"
+#include "phpcommandhandler.h"
+#include "phpcodegenerator.h"
 #include "webinterfacepluginsettings.h"
 
 using namespace bt;
 
 namespace kt
 {
-	
+	QString DataDir();
 	
 
-	HttpServer::HttpServer(CoreInterface *core, int port) : QServerSocket(port, 5),core(core),cache(10,23)
+	HttpServer::HttpServer(CoreInterface *core, bt::Uint16 port) : sock(0),notifier(0),core(core),cache(10),port(port)
 	{
-		php_i = new PhpInterface(core);
-		clients.setAutoDelete(true);
+		qsrand(time(0));
+		sock = new net::Socket(true,4);
+
+		php_cmd = new PhpCommandHandler(core);
+		php_gen = new PhpCodeGenerator(core);
+
 		
-		QStringList dirList = KGlobal::instance()->dirs()->findDirs("data", "ktorrent/www");
-		rootDir = *(dirList.begin());
+		QStringList dirList = KGlobal::dirs()->findDirs("data", "ktorrent/www");
+		rootDir = dirList.front();
 		Out(SYS_WEB|LOG_DEBUG) << "WWW Root Directory "<< rootDir <<endl;
 		session.logged_in = false;
-		cache.setAutoDelete(true);
+		
+		ok = sock->bind(QString::null,port,true);
+		if (ok)
+		{
+			notifier = new QSocketNotifier(sock->fd(),QSocketNotifier::Read,this);
+			connect(notifier,SIGNAL(activated(int)),this,SLOT(slotAccept(int)));
+		}
+
+		skin_list = QDir(rootDir).entryList(QDir::Dirs);
+		skin_list.removeAll("common");
+		skin_list.removeAll(".");
+		skin_list.removeAll("..");
 	}
 	
 	HttpServer::~HttpServer()
 	{
-		delete php_i;
+		sock->close();
+		delete sock;
+		delete php_cmd;
+		delete php_gen;
 	}
 
-	void HttpServer::newConnection(int s)
+	QString HttpServer::skinDir() const
 	{
-		QSocket* socket = new QSocket(this);
-		socket->setSocket(s);
+		QString skin = skin_list[WebInterfacePluginSettings::skin()];
+		if (skin.length() == 0)
+			skin = "default";
+		return rootDir + bt::DirSeparator() + skin;
+	}
 	
-		connect(socket, SIGNAL(readyRead()), this, SLOT(slotSocketReadyToRead()));
-		connect(socket, SIGNAL(delayedCloseFinished()), this, SLOT(slotConnectionClosed()));
-		connect(socket, SIGNAL(connectionClosed()), this, SLOT(slotConnectionClosed()));
+	QString HttpServer::commonDir() const
+	{
+		return rootDir + bt::DirSeparator() + "common";
+	}
+
+	void HttpServer::slotAccept(int )
+	{
+		net::Address addr;
+		int socket = sock->accept(addr);
+		if (socket < 0)
+			return;
 	
 		HttpClientHandler* handler = new HttpClientHandler(this,socket);
-		clients.insert(socket,handler);
-		Out(SYS_WEB|LOG_NOTICE) << "connection from "<< socket->peerAddress().toString()  << endl;
+		connect(handler,SIGNAL(closed()),this,SLOT(slotConnectionClosed()));
+		Out(SYS_WEB|LOG_NOTICE) << "connection from "<< addr.toString()  << endl;
 	}
-	
-
-	void HttpServer::slotSocketReadyToRead()
-	{
-		QSocket* client = (QSocket*)sender();
-		HttpClientHandler* handler = clients.find(client);
-		if (!handler)
-		{
-			client->deleteLater();
-			return;
-		}
-		
-		handler->readyToRead();
-	}
-	
+/*
 	static int DecodeEscapedChar(QString & password,int idx)
 	{
-		QChar a = password[idx + 1].lower();
-		QChar b = password[idx + 2].lower();
-		if (!a.isNumber() && !(a.latin1() >= 'a' && a.latin1() <= 'f'))
+		QChar a = password[idx + 1].toLower();
+		QChar b = password[idx + 2].toLower();
+		if (!a.isNumber() && !(a.toLatin1() >= 'a' && a.toLatin1() <= 'f'))
 			return idx + 2; // not a valid hex digit
 		
-		if (!b.isNumber() && !(b.latin1() >= 'a' && b.latin1() <= 'f'))
+		if (!b.isNumber() && !(b.toLatin1() >= 'a' && b.toLatin1() <= 'f'))
 			return idx + 2; // not a valid hex digit
 		
 		// calculate high and low nibble
-		Uint8 h = (a.latin1() - (a.isNumber() ? '0' : 'a')) << 4;
-		Uint8 l = (b.latin1() - (b.isNumber() ? '0' : 'a'));
+		Uint8 h = (a.toLatin1() - (a.isNumber() ? '0' : 'a')) << 4;
+		Uint8 l = (b.toLatin1() - (b.isNumber() ? '0' : 'a'));
 		char r = (char) h | l; // combine them and cast to a char
 		password.replace(idx,3,r);
 		return idx + 1;
 	}
-	
+	*/
 	bool HttpServer::checkLogin(const QHttpRequestHeader & hdr,const QByteArray & data)
 	{
 		if (hdr.contentType() != "application/x-www-form-urlencoded")
+		{
+			challenge = QString();
 			return false;
+		}
 		
 		QString username;
-		QString password;
-		QStringList params = QStringList::split("&",QString(data));
+		QString challenge_hash;
+		QStringList params = QString(data).split("&");
 		for (QStringList::iterator i = params.begin();i != params.end();i++)
 		{
 			QString t = *i;
 			if (t.section("=",0,0) == "username")
 				username = t.section("=",1,1);
-			else if (t.section("=",0,0) == "password")
-				password = t.section("=",1,1);
-			
-			// check for passwords with url encoded stuff in them and decode them if necessary
-			int idx = 0;
-			while ((idx = password.find('%',idx)) > 0)
-			{
-				if (idx + 2 < password.length())
-				{
-					idx = DecodeEscapedChar(password,idx);
-				}
-				else
-					break;
-			}
+			else if (t.section("=",0,0) == "challenge")
+				challenge_hash = t.section("=",1,1);
 		}
 
-		if (!username.isNull() && !password.isNull())
+		if (username.isNull() || challenge.isNull() || username != WebInterfacePluginSettings::username())
 		{
-			KMD5 context(password.utf8());
-
-			if(username == WebInterfacePluginSettings::username() && 
-				context.hexDigest().data() == WebInterfacePluginSettings::password())
-			{
-				session.logged_in = true;
-				session.sessionId=rand();
-				session.last_access=QTime::currentTime();
-				Out(SYS_WEB|LOG_NOTICE) << "Webgui login succesfull !" << endl;
-				return true;
-			}
+			challenge = QString();
+			return false;
 		}
-
+		
+		QByteArray s = (challenge + WebInterfacePluginSettings::password()).toUtf8();
+		bt::SHA1Hash hash = bt::SHA1Hash::generate((const bt::Uint8*)s.data(),s.length());
+		if (hash.toString() == challenge_hash)
+		{
+			session.logged_in = true;
+			session.sessionId=rand();
+			session.last_access=QTime::currentTime();
+			Out(SYS_WEB|LOG_NOTICE) << "Webgui login succesfull !" << endl;
+			challenge = QString();
+			return true;
+		}
+		challenge = QString();
 		return false;
 	}
 	
@@ -170,7 +184,7 @@ namespace kt
 		if (hdr.hasKey("Cookie"))
 		{
 			QString cookie = hdr.value("Cookie");
-			int idx = cookie.find("KT_SESSID=");
+			int idx = cookie.indexOf("KT_SESSID=");
 			if (idx == -1)
 				return false;
 			
@@ -222,33 +236,33 @@ namespace kt
 			return QString::null;
 	}
 	
-	// HTTP needs non translated dates
+		// HTTP needs non translated dates
 	static QString days[] = {
 		"Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"
 	};
-	
+		
 	static QString months[] = {
 		"Jan","Feb","Mar","Apr",
 		"May","Jun","Jul","Aug",
 		"Sep","Oct","Nov","Dec"
 	};
-	
+		
 	static QString DateTimeToString(const QDateTime & now,bool cookie)
 	{
 		if (!cookie)
 			return now.toString("%1, dd %2 yyyy hh:mm:ss UTC")
-				.arg(days[now.date().dayOfWeek() - 1])
-				.arg(months[now.date().month() - 1]);
+					.arg(days[now.date().dayOfWeek() - 1])
+					.arg(months[now.date().month() - 1]);
 		else
 			return now.toString("%1, dd-%2-yyyy hh:mm:ss GMT")
-				.arg(days[now.date().dayOfWeek() - 1])
-				.arg(months[now.date().month() - 1]);
+					.arg(days[now.date().dayOfWeek() - 1])
+					.arg(months[now.date().month() - 1]);
 	}
 		
 	void HttpServer::setDefaultResponseHeaders(HttpResponseHeader & hdr,const QString & content_type,bool with_session_info)
 	{
 		hdr.setValue("Server","KTorrent/" KT_VERSION_MACRO);
-		hdr.setValue("Date",DateTimeToString(QDateTime::currentDateTime(Qt::UTC),false));
+		hdr.setValue("Date",DateTimeToString(QDateTime::currentDateTime().toUTC(),false));
 		hdr.setValue("Content-Type",content_type);
 		hdr.setValue("Connection","keep-alive");
 		if (with_session_info && session.sessionId && session.logged_in)
@@ -262,7 +276,7 @@ namespace kt
 		HttpResponseHeader rhdr(301);
 		setDefaultResponseHeaders(rhdr,"text/html",false);
 		rhdr.setValue("Location","/login.html");
-		QString path = rootDir + bt::DirSeparator() + WebInterfacePluginSettings::skin() + "/login.html";
+		QString path = skinDir() + "/login.html";
 		if (!hdlr->sendFile(rhdr,path))
 		{
 			HttpResponseHeader nhdr(404);
@@ -280,11 +294,15 @@ namespace kt
 		
 		//Out(SYS_WEB|LOG_DEBUG) << "GET " << hdr.path() << endl;
 		
-		KURL url;
+		KUrl url;
 		url.setEncodedPathAndQuery(file);
 		
-		QString path = rootDir + bt::DirSeparator() + WebInterfacePluginSettings::skin() + url.path();
-		// first check if the file exists (if not send 404)
+		QString path = commonDir() + url.path();
+		// first try the common dir
+		if (!bt::Exists(path)) // doesn't exist so it must be in the skin dir
+			path = skinDir() + url.path();
+		
+		// check if the file exists (if not send 404)
 		if (!bt::Exists(path))
 		{
 			HttpResponseHeader rhdr(404);
@@ -294,14 +312,14 @@ namespace kt
 		}
 		
 		QFileInfo fi(path);
-		QString ext = fi.extension();
+		QString ext = fi.suffix();
 		
 		// if it is the login page send that
 		if (file == "/login.html" || file == "/")
 		{
 			session.logged_in = false;
 			ext = "html";
-			path = rootDir + bt::DirSeparator() + WebInterfacePluginSettings::skin() + "/login.html"; 
+			path = skinDir() + "/login.html"; 
 		}
 		else if (!session.logged_in && (ext == "html" || ext == "php"))
 		{
@@ -351,7 +369,7 @@ namespace kt
 					setDefaultResponseHeaders(rhdr,"text/html",true);
 					rhdr.setValue("Cache-Control","max-age=0");
 					rhdr.setValue("Last-Modified",DateTimeToString(fi.lastModified(),false));
-					rhdr.setValue("Expires",DateTimeToString(QDateTime::currentDateTime(Qt::UTC).addSecs(3600),false));
+					rhdr.setValue("Expires",DateTimeToString(QDateTime::currentDateTime().toUTC().addSecs(3600),false));
 					hdlr->sendResponse(rhdr);
 					return;
 				}
@@ -361,7 +379,7 @@ namespace kt
 			HttpResponseHeader rhdr(200);
 			setDefaultResponseHeaders(rhdr,ExtensionToContentType(ext),true);
 			rhdr.setValue("Last-Modified",DateTimeToString(fi.lastModified(),false));
-			rhdr.setValue("Expires",DateTimeToString(QDateTime::currentDateTime(Qt::UTC).addSecs(3600),false));
+			rhdr.setValue("Expires",DateTimeToString(QDateTime::currentDateTime().toUTC().addSecs(3600),false));
 			rhdr.setValue("Cache-Control","private");
 			if (!hdlr->sendFile(rhdr,path))
 			{
@@ -372,10 +390,11 @@ namespace kt
 		}
 		else if (ext == "php")
 		{
+			QMap<QString,QString> args = url.queryItems();
 			bool redirect = false;
 			bool shutdown = false;
-			if (url.queryItems().count() > 0 && session.logged_in)
-				redirect = php_i->exec(url,shutdown);
+			if (args.count() > 0 && session.logged_in)
+				redirect = php_cmd->exec(url,shutdown);
 			
 			if (shutdown)
 			{
@@ -389,7 +408,7 @@ namespace kt
 				setDefaultResponseHeaders(rhdr,"text/html",true);
 				rhdr.setValue("Location",url.encodedPathAndQuery());
 				
-				hdlr->executePHPScript(php_i,rhdr,WebInterfacePluginSettings::phpExecutablePath(),
+				hdlr->executePHPScript(php_gen,rhdr,WebInterfacePluginSettings::phpExecutablePath().path(),
 									   path,url.queryItems());
 			}
 			else
@@ -397,7 +416,7 @@ namespace kt
 				HttpResponseHeader rhdr(200);
 				setDefaultResponseHeaders(rhdr,"text/html",true);
 			
-				hdlr->executePHPScript(php_i,rhdr,WebInterfacePluginSettings::phpExecutablePath(),
+				hdlr->executePHPScript(php_gen,rhdr,WebInterfacePluginSettings::phpExecutablePath().path(),
 								   path,url.queryItems());
 			}
 		}
@@ -431,9 +450,9 @@ namespace kt
 	void HttpServer::handleTorrentPost(HttpClientHandler* hdlr,const QHttpRequestHeader & hdr,const QByteArray & data)
 	{
 		const char* ptr = data.data();
-		Uint32 len = data.size();
-		int pos = QString(data).find("\r\n\r\n");
-
+		int len = data.size();
+		int pos = QString(data).indexOf("\r\n\r\n");
+		
 		if (!session.logged_in || !checkSession(hdr))
 		{
 			// You can't post torrents if you are not logged in
@@ -442,7 +461,7 @@ namespace kt
 			return;
 		}
 		
-		if (pos == -1 || pos + 4 >= len || ptr[pos + 4] != 'd')
+		if (pos == -1 || pos + 4 >= len)
 		{
 			HttpResponseHeader rhdr(500);
 			setDefaultResponseHeaders(rhdr,"text/html",false);
@@ -451,9 +470,10 @@ namespace kt
 		}
 		
 		// save torrent to a temporary file
-		KTempFile tmp_file(locateLocal("tmp", "ktwebgui-"), ".torrent");
-		QDataStream* out = tmp_file.dataStream();
-		if (!out)
+		QString save_file = kt::DataDir() + "webgui_load_torrent";
+		QFile tmp_file(save_file);
+		
+		if (!tmp_file.open(QIODevice::WriteOnly))
 		{
 			HttpResponseHeader rhdr(500);
 			setDefaultResponseHeaders(rhdr,"text/html",false);
@@ -461,12 +481,13 @@ namespace kt
 			return;
 		}
 		
-		out->writeRawBytes(ptr + (pos + 4),len - (pos + 4));
-		tmp_file.sync();
-		tmp_file.setAutoDelete(true);
+		QDataStream out(&tmp_file);
+		out.writeRawData(ptr + (pos + 4),len - (pos + 4));
+		out << flush;
+		tmp_file.close();
 		
-		Out(SYS_WEB|LOG_NOTICE) << "Loading file " << tmp_file.name() << endl;
-		core->loadSilently(KURL::fromPathOrURL(tmp_file.name()));
+		Out(SYS_WEB|LOG_NOTICE) << "Loading file " << save_file << endl;
+		core->loadSilently(KUrl(save_file),QString());
 		
 		handleGet(hdlr,hdr);
 	}
@@ -480,8 +501,8 @@ namespace kt
 	
 	void HttpServer::slotConnectionClosed()
 	{
-		QSocket* socket= (QSocket*)sender();
-		clients.erase(socket);
+		HttpClientHandler* client = (HttpClientHandler*)sender();
+		client->deleteLater();
 	}
 
 	QDateTime HttpServer::parseDate(const QString & str)
@@ -492,7 +513,7 @@ namespace kt
       		Sunday, 06-Nov-94 08:49:37 GMT ; RFC 850, obsoleted by RFC 1036
       		Sun Nov  6 08:49:37 1994       ; ANSI C's asctime() format
 		*/
-		QStringList sl = QStringList::split(" ",str);
+		QStringList sl = str.split(" ");
 		if (sl.count() == 6)
 		{
 			// RFC 1123 format
@@ -511,7 +532,7 @@ namespace kt
 		else if (sl.count() == 4)
 		{
 			//  RFC 1036
-			QStringList dl = QStringList::split("-",sl[1]);
+			QStringList dl = sl[1].split("-");
 			if (dl.count() != 3)
 				return QDateTime();
 			
@@ -548,14 +569,33 @@ namespace kt
 	
 	bt::MMapFile* HttpServer::cacheLookup(const QString & name)
 	{
-		return cache.find(name);
+		return cache.object(name);
 	}
 	
 	void HttpServer::insertIntoCache(const QString & name,bt::MMapFile* file)
 	{
 		cache.insert(name,file);
 	}
+	
+	static char RandomLetterOrNumber()
+	{
+		int i = qrand() % 62;
+		if (i < 26)
+			return 'a' + i;
+		else if (i < 52)
+			return 'A' + (i - 26);
+		else
+			return '0' + (i - 52);
+	}
 
+	QString HttpServer::challengeString()
+	{
+		// regenerate challenge string
+		challenge = QString();
+		for (int i = 0;i < 20;i++)
+			challenge.append(RandomLetterOrNumber());
+		return challenge;
+	}
 }
 
 #include "httpserver.moc"
