@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005 by Joris Guisson                                   *
+ *   Copyright (C) 2005-2007 by Joris Guisson                              *
  *   joris.guisson@gmail.com                                               *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -21,14 +21,12 @@
 #include <util/log.h>
 #include <interfaces/coreinterface.h>
 #include <interfaces/torrentinterface.h>
-#include <torrent/globals.h>
-#include <torrent/server.h>
+#include <torrent/queuemanager.h>
 #include "zeroconfplugin.h"
-#include "avahiservice.h"
+#include "torrentservice.h"
+
 		
 		
-#define NAME "Zeroconf"
-#define AUTHOR "Lesly Weyts and Kevin Andre"
 		
 K_EXPORT_COMPONENT_FACTORY(ktzeroconfplugin,KGenericFactory<kt::ZeroConfPlugin>("ktzeroconfplugin"))
 
@@ -37,9 +35,9 @@ using namespace bt;
 namespace kt
 {
 
-	ZeroConfPlugin::ZeroConfPlugin(QObject* parent, const char* name, const QStringList& args)
-	: Plugin(parent, name,args,NAME,i18n("Zeroconf"),AUTHOR,QString::null,i18n("Finds peers running ktorrent on the local network to share torrents with"),"ktplugins")
+	ZeroConfPlugin::ZeroConfPlugin(QObject* parent,const QStringList& args) : Plugin(parent)
 	{
+		Q_UNUSED(args);
 		services.setAutoDelete(true);
 	}
 
@@ -50,14 +48,14 @@ namespace kt
 	void ZeroConfPlugin::load()
 	{
 		CoreInterface* core = getCore();
-		connect(core,SIGNAL(torrentAdded( kt::TorrentInterface* )),
-				this,SLOT(torrentAdded( kt::TorrentInterface* )));
-		connect(core,SIGNAL(torrentRemoved( kt::TorrentInterface* )),
-				this,SLOT(torrentRemoved( kt::TorrentInterface* )));
+		connect(core,SIGNAL(torrentAdded( bt::TorrentInterface* )),
+				this,SLOT(torrentAdded( bt::TorrentInterface* )));
+		connect(core,SIGNAL(torrentRemoved( bt::TorrentInterface* )),
+				this,SLOT(torrentRemoved( bt::TorrentInterface* )));
 		
 		// go over existing torrents and add them
-		bt::QueueManager* qman = core->getQueueManager();
-		for (QPtrList<kt::TorrentInterface>::iterator i = qman->begin();i != qman->end();i++)
+		kt::QueueManager* qman = core->getQueueManager();
+		for (QList<bt::TorrentInterface*>::iterator i = qman->begin();i != qman->end();i++)
 		{
 			torrentAdded(*i);
 		}
@@ -66,41 +64,40 @@ namespace kt
 	void ZeroConfPlugin::unload()
 	{
 		CoreInterface* core = getCore();
-		disconnect(core,SIGNAL(torrentAdded( kt::TorrentInterface* )),
-				   this,SLOT(torrentAdded( kt::TorrentInterface* )));
-		disconnect(core,SIGNAL(torrentRemoved( kt::TorrentInterface* )),
-				   this,SLOT(torrentRemoved( kt::TorrentInterface*)));
+		disconnect(core,SIGNAL(torrentAdded( bt::TorrentInterface* )),
+				   this,SLOT(torrentAdded( bt::TorrentInterface* )));
+		disconnect(core,SIGNAL(torrentRemoved( bt::TorrentInterface* )),
+				   this,SLOT(torrentRemoved( bt::TorrentInterface*)));
 		
-		bt::PtrMap<kt::TorrentInterface*,AvahiService>::iterator i = services.begin();
+		bt::PtrMap<bt::TorrentInterface*,TorrentService>::iterator i = services.begin();
 		while (i != services.end())
 		{
-			AvahiService* av = i->second;
-			kt::TorrentInterface* ti = i->first;
+			TorrentService* av = i->second;
+			bt::TorrentInterface* ti = i->first;
 			ti->removePeerSource(av);
 			i++;
 		}
 		services.clear();
 	}
 	
-	void ZeroConfPlugin::torrentAdded(kt::TorrentInterface* tc)
+	void ZeroConfPlugin::torrentAdded(bt::TorrentInterface* tc)
 	{
 		if (services.contains(tc))
 			return;
 		
-		bt::Uint16 port = bt::Globals::instance().getServer().getPortInUse();
-		AvahiService* av = new AvahiService(tc->getOwnPeerID(),port,tc->getInfoHash());
+		
+		TorrentService* av = new TorrentService(tc);
 		services.insert(tc,av);
 		tc->addPeerSource(av);
-		Out(SYS_ZCO|LOG_NOTICE) << "ZeroConf service added for " 
-				<< tc->getStats().torrent_name << endl;
-		connect(av,SIGNAL(serviceDestroyed( AvahiService* )),
-				this,SLOT(avahiServiceDestroyed( AvahiService* )));
+		Out(SYS_ZCO|LOG_NOTICE) << "ZeroConf service added for " << tc->getStats().torrent_name << endl;
+		connect(av,SIGNAL(serviceDestroyed( TorrentService* )),
+			this,SLOT(avahiServiceDestroyed( TorrentService* )));
 	}
 
 		
-	void ZeroConfPlugin::torrentRemoved(kt::TorrentInterface* tc)
+	void ZeroConfPlugin::torrentRemoved(bt::TorrentInterface* tc)
 	{
-		AvahiService* av = services.find(tc);
+		TorrentService* av = services.find(tc);
 		if (!av)
 			return;
 		Out(SYS_ZCO|LOG_NOTICE) << "ZeroConf service removed for " 
@@ -109,12 +106,12 @@ namespace kt
 		services.erase(tc);
 	}
 
-	void ZeroConfPlugin::avahiServiceDestroyed(AvahiService* av)
+	void ZeroConfPlugin::avahiServiceDestroyed(TorrentService* av)
 	{
 		services.setAutoDelete(false);
 		
 		Out(SYS_ZCO|LOG_NOTICE) << "ZeroConf service destroyed " << endl;
-		bt::PtrMap<kt::TorrentInterface*,AvahiService>::iterator i = services.begin();
+		bt::PtrMap<bt::TorrentInterface*,TorrentService>::iterator i = services.begin();
 		while (i != services.end())
 		{
 			if (i->second == av)
